# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 

from platform import system
import ctypes
if system()=='Darwin' or system()=='Linux':
	fdll=ctypes.CDLL
	typesizet=ctypes.c_ulong
	npulong='=u8'
elif system()=='Windows':
	fdll=ctypes.WinDLL
	typesizet=ctypes.c_ulonglong
	npulong='=u4'
else:
	raise OSError("Unsupported operating system.")
